<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\DependencyInjection;

use Dedi\SyliusQuotePlugin\Processor\Quote\QuoteProcessorInterface;
use Dedi\SyliusQuotePlugin\Service\QuoteConfiguration;
use Dedi\SyliusQuotePlugin\Service\QuoteConfigurationInterface;
use Sylius\Bundle\ResourceBundle\DependencyInjection\Extension\AbstractResourceExtension;
use Symfony\Component\Config\Definition\ConfigurationInterface;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Loader\XmlFileLoader;

final class DediSyliusQuoteExtension extends AbstractResourceExtension
{
    public function load(array $configs, ContainerBuilder $container): void
    {
        $config = $this->processConfiguration($this->getConfiguration([], $container), $configs);
        $loader = new XmlFileLoader($container, new FileLocator(__DIR__ . '/../Resources/config'));
        $this->registerResources('dedi_quote', $config['driver'], $config['resources'], $container);

        $loader->load('services.xml');

        $container
            ->register('dedi_quote.configuration', QuoteConfiguration::class)
            ->setArgument('$anonymousQuoteAllowed', $config['allow_anonymous_quote'])
            ->setArgument('$cartFromQuoteEditAllowed', $config['allow_cart_edit'])
            ->setArgument('$orderFromQuoteCancelAllowed', $config['allow_order_cancel'])
        ;
        $container
            ->register(QuoteConfigurationInterface::class, QuoteConfiguration::class)
        ;

        $container
            ->registerForAutoconfiguration(QuoteProcessorInterface::class)
            ->addTag(RegisterProcessorsPass::PROCESSOR_SERVICE_TAG)
        ;
    }

    public function getConfiguration(array $config, ContainerBuilder $container): ConfigurationInterface
    {
        return new Configuration();
    }
}
