<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\EventListener\Sales;

use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Service\QuoteConfigurationAwareInterface;
use Dedi\SyliusQuotePlugin\Service\QuoteConfigurationInterface;
use Dedi\SyliusQuotePlugin\Specification\Order\IsFromQuoteSpecification;
use InvalidArgumentException;
use SM\Factory\FactoryInterface as StateMachineFactoryInterface;
use Sylius\Bundle\ResourceBundle\Event\ResourceControllerEvent;
use Sylius\Component\Order\OrderTransitions;

class OrderQuoteLockListener implements QuoteConfigurationAwareInterface
{
    private QuoteConfigurationInterface $quoteConfiguration;

    private IsFromQuoteSpecification $isFromQuoteSpecification;

    private StateMachineFactoryInterface $stateMachineFactory;

    public function __construct(
        QuoteConfigurationInterface $quoteConfiguration,
        IsFromQuoteSpecification $isFromQuoteSpecification,
        StateMachineFactoryInterface $stateMachineFactory,
    ) {
        $this->quoteConfiguration = $quoteConfiguration;
        $this->isFromQuoteSpecification = $isFromQuoteSpecification;
        $this->stateMachineFactory = $stateMachineFactory;
    }

    public function __invoke(ResourceControllerEvent $event): void
    {
        $stateMachine = $this->stateMachineFactory->get($event->getSubject(), 'sylius_order');

        if ($this->quoteConfiguration->isCartFromQuoteEditAllowed() && $stateMachine->can(OrderTransitions::TRANSITION_CREATE)) {
            return;
        }

        if ($this->quoteConfiguration->isOrderFromQuoteCancelAllowed() && $stateMachine->can(OrderTransitions::TRANSITION_CANCEL)) {
            return;
        }

        $order = $event->getSubject();

        if (!$order instanceof OrderInterface) {
            throw new InvalidArgumentException(sprintf(
                'Expected an instance of "%s", got "%s"',
                OrderInterface::class,
                get_class($order),
            ));
        }

        if ($this->isFromQuoteSpecification->isSatisfiedBy($order)) {
            $event->stop('dedi_quote.cart.cannot_update_from_quote');
        }
    }

    public function getQuoteConfiguration(): QuoteConfigurationInterface
    {
        return $this->quoteConfiguration;
    }
}
