<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Service;

use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Enum\QuoteTransitionEnum;
use Dedi\SyliusQuotePlugin\Specification\Order\IsQuoteSpecification;
use Doctrine\ORM\EntityManagerInterface;
use SM\Factory\Factory;

class QuoteValidatedStateApplicator
{
    private readonly Factory $stateMachineFactory;

    private readonly EntityManagerInterface $entityManager;

    private readonly IsQuoteSpecification $isQuoteSpecification;

    public function __construct(
        Factory $stateMachineFactory,
        EntityManagerInterface $entityManager,
        IsQuoteSpecification $isQuoteSpecification,
    ) {
        $this->stateMachineFactory = $stateMachineFactory;
        $this->entityManager = $entityManager;
        $this->isQuoteSpecification = $isQuoteSpecification;
    }

    public function apply(OrderInterface $order): void
    {
        if (!$this->isQuoteSpecification->isSatisfiedBy($order)) {
            return;
        }

        $stateMachine = $this->stateMachineFactory->get($order, QuoteTransitionEnum::GRAPH);

        $stateMachine->apply(QuoteTransitionEnum::TRANSITIONS_VALIDATE);

        $this->entityManager->persist($order);
    }
}
